/*
** Module   : LowLevelSocketListener.java
** Abstract : An interface holding the SERVER-SOCKET client specific api.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CS  20130425 Created initial version.
*/

package com.goldencode.p2j.util;

import java.io.*;
import java.security.*;
import java.security.cert.*;

/**
 * Interface for Server-Socket methods which require to be implemented on the client
 *
 */
public interface LowLevelSocketListener
{

   /**
    * Client side implementation for the ENABLE-CONNECTION() SERVER-SOCKET method using regular
    * Server sockets, this will attempt to start listening  on the specified port with the given 
    * timeout and maximum number of connections.
    * 
    * @param  port
    *         The port in which it will listen
    * @param  timeout
    *         The timeout for a connection in ms
    * @param  maxConn
    *         The maximum number of connections to accept
    * @throws IOException
    */
   public void enableConnections(int port, int timeout, int maxConn)
   throws IOException;

   /**
    * Client side implementation for the ENABLE-CONNECTION() SERVER-SOCKET method using SSL
    * Server sockets, this will attempt to start listening  on the specified port with the given 
    * timeout and maximum number of connections.
    * 
    * @param  port
    *         The port in which it will listen
    * @param  timeout
    *         The timeout for a connection in ms
    * @param  maxConn
    *         The maximum number of connections to accept
    * @param  keyInput
    *         The Input Stream used to get the certificate from
    * @param  keyPassword
    *         The password for the certificate
    *         
    * @throws KeyManagementException
    *         If operation fails.
    * @throws NoSuchAlgorithmException
    *         If operation fails.
    * @throws UnrecoverableKeyException
    *         If operation fails.
    * @throws KeyStoreException
    *         If operation fails.
    * @throws CertificateException
    *         If operation fails.
    */
   public void enableSSLConnections(int         port,
                                    int         timeout,
                                    int         maxConn,
                                    InputStream keyInput,
                                    char[]      keyPassword)
   throws KeyManagementException,
          NoSuchAlgorithmException,
          UnrecoverableKeyException,
          KeyStoreException,
          CertificateException;
   
   /**
    * Start listening for connections.
    * 
    * @return the wrapped accepted socket 
    * 
    * @throws IOException
    *         In case it fails
    */
   // TODO decide how we handle an accepted connection 
   // should we start the listening Thread on the client?
   public LowLevelSocket accept() 
   throws IOException;

}
